package body Stack
--# own State is CState; -- rappresentazione

is

   subtype PointerT is Natural range Natural'First .. MaxSize;

   type IntVectorT is array (Integer range <>) of Integer;
   subtype StackRange is Positive range 1 .. MaxSize;
   subtype IntStackT is IntVectorT (StackRange);

   EmptyVector : constant IntStackT := IntStackT'(StackRange => 0);

   type StackCT is record
      Pointer : PointerT;
      Vector : IntStackT;
   end record;

   CState : StackCT;

   -----------
   -- Empty --
   -----------

   function Empty return Boolean
   --# global CState;
   --# return ( CState.Pointer = 0 );
   is
   begin
      return CState.Pointer = 0;
   end Empty;

   -----------
   -- Clear --
   -----------

   --# function zeros(V: IntStackT) return Boolean;
   --  Def zeros(V) <->  ( for all I in IntStackT => (V(I) = 0) );
   --  R_zeros : zeros(EmptyVector) <-> true;

   procedure Clear
   --# global out CState;
   --# derives CState from ;
   --# post CState.Pointer = 0;
   is
   begin
      CState.Pointer := 0;
      CState.Vector  := EmptyVector;
   end Clear;

   ---------
   -- Pop --
   ---------

   procedure Pop (X : out Integer)
   --# global in out CState;
   --# derives CState from *
   --# &       X from CState;
   --# pre  CState.Pointer > 0 ;
   --# post X = CState~.Vector(CState~.Pointer) and
   --#      CState.Pointer = CState~.Pointer - 1 ;
   -- #      and ( for all I in StackRange range StackRange'First .. Pointer - 1
   -- #            => ( Vector(I) = Vector(I) )) ;
   --  servirebbe, perch Vector non  pi protetto
   is
   begin
      X := CState.Vector(CState.Pointer);
      CState.Pointer := CState.Pointer - 1;
   end Pop;

   ----------
   -- Push --
   ----------

   procedure Push (Y : in Integer)
   --# global in out CState;
   --# derives CState from *, Y;
   --# pre  CState.Pointer < StackRange'Last;
   --# post CState.Pointer = CState~.Pointer + 1 and CState.Vector(CState.Pointer) = Y and
   --#      ( for all I in StackRange range StackRange'First .. CState~.Pointer
   --#            => ( CState.Vector(I) = CState~.Vector(I) )) ;

   is
   begin
      CState.Pointer := CState.Pointer + 1;
      CState.Vector(CState.Pointer) := Y;
   end Push;

begin
      CState.Pointer := 0;
      CState.Vector  := EmptyVector;
end Stack;
